require "collisions"

local fireslime = {}
fireslime.__index = fireslime

function newFireslime(object)
	local n = object
	n.type = "fireslime"
	n.width = 16
	n.height = 16
	n.xspeed = 0
	n.yspeed = 0
	n.yaccel = 0.075
	n.behavior = "sleeping"
	n.stance = "sleep"
	n.direction = "left"
	n.WAIT = 0
	n.HIT = 0
	n.hp = 20
	n.DIYING = 0
	n.animations = {
		sleep = {
			left  = newAnimation(lutro.graphics.newImage(
				"assets/fireslime_sleep_left.png"),  16, 16, 2, 10),
			right = newAnimation(lutro.graphics.newImage(
				"assets/fireslime_sleep_right.png"), 16, 16, 2, 10)
		},
		awake = {
			left  = newAnimation(lutro.graphics.newImage(
				"assets/fireslime_awake_left.png"),  16, 16, 2, 10),
			right = newAnimation(lutro.graphics.newImage(
				"assets/fireslime_awake_right.png"), 16, 16, 2, 10)
		},
		hit = {
			left  = newAnimation(lutro.graphics.newImage(
				"assets/fireslime_hit_left.png"),  16, 16, 2, 10),
			right = newAnimation(lutro.graphics.newImage(
				"assets/fireslime_hit_right.png"), 16, 16, 2, 10)
		},
		die = {
			left  = newAnimation(lutro.graphics.newImage(
				"assets/fireslime_die_left.png"),  16, 16, 2, 60),
			right = newAnimation(lutro.graphics.newImage(
				"assets/fireslime_die_right.png"), 16, 16, 2, 60)
		},
	}

	n.anim = n.animations[n.stance][n.direction]

	return setmetatable(n, fireslime)
end

function fireslime:on_the_ground()
	return solid_at(self.x + 8, self.y + 16, self)
end

function fireslime:update(dt)
	local otg = self:on_the_ground()

	if self.hp <= 0 then
		self.xspeed = 0
		self.behavior = "sleeping"
	end

	-- gravity
	if not otg then
		self.yspeed = self.yspeed + self.yaccel
		if (self.yspeed > 3) then self.yspeed = 3 end
		self.y = self.y + self.yspeed
	elseif self.HIT == 0 then
		self.xspeed = 0
	end

	local dX = (self.x + self.width/2) - (character.x + character.width/2)
	local dY = (self.y + self.height/2) - (character.y + character.height/2)
	local distance = math.sqrt( ( dX^2 ) + ( dY^2 ) )

	if distance < 64 and self.behavior == "sleeping" and self.hp > 0 then
		self.behavior = "awake"
	end

	if self.behavior == "awake" and otg and self.hp > 0 then
		if dX > 0 then
			self.direction = "left"
		else
			self.direction = "right"
		end
	end

	if self.behavior == 'awake' and otg and distance < 256 and self.WAIT == 0 and self.hp > 0 then
		if self.direction == "left" then
			self.xspeed = -1
		else
			self.xspeed = 1
		end
		self.y = self.y - 1
		self.yspeed = -1.5
		self.WAIT = 64
	end

	self.x = self.x + self.xspeed
	self.y = self.y + self.yspeed

	if self.HIT > 0 then
		if self.xspeed > 0 then
			self.xspeed = self.xspeed - 0.05
			if self.xspeed < 0 then
				self.xspeed = 0
			end
		elseif self.xspeed < 0 then
			self.xspeed = self.xspeed + 0.05;
			if self.xspeed > 0 then
				self.xspeed = 0
			end
		end
	end

	-- animations
	if self.DIYING > 0 then
		self.stance = "die"
	elseif self.HIT > 0 then
		self.stance = "hit"
	elseif self.behavior == "sleeping" then
		self.stance = "sleep"
	else
		self.stance = "awake"
	end

	local anim = self.animations[self.stance][self.direction]
	-- always animate from first frame 
	if anim ~= self.anim then
		anim.timer = 0
	end
	self.anim = anim;

	self.anim:update(dt)

	if self.WAIT > 0 then
		self.WAIT = self.WAIT - 1
	end

	if self.HIT > 0 then
		self.HIT = self.HIT - 1
	end

	if self.DIYING > 0 then
		self.DIYING = self.DIYING - 1
	end

	if self.DIYING == 1 then
		for i=1, #entities do
			if entities[i] == self then
				table.remove(entities, i)
			end
		end
	end
end

function fireslime:draw()
	self.anim:draw(self.x, self.y)
end

function fireslime:on_collide(e1, e2, dx, dy)
	if e2.type == "ground" then
		if math.abs(dy) < math.abs(dx) and dy ~= 0 then
			self.yspeed = 0
			self.y = self.y + dy
		end

		if math.abs(dx) < math.abs(dy) and dx ~= 0 then
			self.xspeed = 0
			self.x = self.x + dx
		end
	elseif (e2.type == "sword" or e2.type == "magicarrow") and self.HIT == 0 and self.hp > 0 then
		local dmg = 4
		self.HIT = 32
		self.WAIT = 64
		self.y = self.y - 1
		self.yspeed = -1
		if character.x < self.x then
			self.xspeed = 2
		else
			self.xspeed = -2
		end
		self.behavior = "awake"
		lutro.audio.play(sfx_fireslimehit)
		table.insert(entities, newNotif({x=self.x, y=self.y, text=dmg}))
		self.hp = self.hp - dmg
		if self.hp <= 0 then
			lutro.audio.play(sfx_fatknightdie)
			self.DIYING = 64
		end
	end
end